try:
    import json
except ImportError:
    from django.utils import simplejson as json

from respite.serializers.jsonserializer import JSONSerializer

class JSONPSerializer(JSONSerializer):

    def serialize(self, request):
        data = super(JSONPSerializer, self).serialize(request)

        if 'callback' in request.GET:
            callback = request.GET['callback']
        else:
            callback = 'callback'

        return '%s(%s)' % (callback, data)



import inspect

from functools import wraps

from django.http import HttpResponse

from respite.urls import routes

def override_supported_formats(formats):
    """
    Override the views class' supported formats for the decorated function.

    Arguments:
    formats -- A list of strings describing formats, e.g. ``['html', 'json']``.
    """
    def decorator(function):
        @wraps(function)
        def wrapper(self, *args, **kwargs):
            self.supported_formats = formats
            return function(self, *args, **kwargs)
        return wrapper
    return decorator

def route(regex, method, name):
    """
    Route the decorated view.

    :param regex: A string describing a regular expression to which the request path will be matched.
    :param method: A string describing the HTTP method that this view accepts.
    :param name: A string describing the name of the URL pattern.
    
    ``regex`` may also be a lambda that accepts the parent resource's ``prefix`` argument and returns
    a string describing a regular expression to which the request path will be matched.
    
    ``name`` may also be a lambda that accepts the parent resource's ``views`` argument and returns
    a string describing the name of the URL pattern.
    """

    def decorator(function):
        function.route = routes.route(
            regex = regex,
            view = function.__name__,
            method = method,
            name = name
        )

        @wraps(function)
        def wrapper(self, *args, **kwargs):
            return function(self, *args, **kwargs)
        return wrapper

    return decorator

def before(method_name):
    """
    Run the given method prior to the decorated view.

    If you return anything besides ``None`` from the given method,
    its return values will replace the arguments of the decorated
    view.

    If you return an instance of ``HttpResponse`` from the given method,
    Respite will return it immediately without delegating the request to the
    decorated view.

    Example usage::

        class ArticleViews(Views):

            @before('_load')
            def show(self, request, article):
                return self._render(
                    request = request,
                    template = 'show',
                    context = {
                        'article': article
                    }
                )

            def _load(self, request, id):
                try:
                    return request, Article.objects.get(id=id)
                except Article.DoesNotExist:
                    return self._error(request, 404, message='The article could not be found.')

    :param method: A string describing a class method.
    """
    def decorator(function):
        @wraps(function)
        def wrapper(self, *args, **kwargs):
            returns = getattr(self, method_name)(*args, **kwargs)

            if returns is None:
                return function(self, *args, **kwargs)
            else:
                if isinstance(returns, HttpResponse):
                    return returns
                else:
                    return function(self, *returns)
        return wrapper
    return decorator



import re

from urllib import urlencode

from django.http import QueryDict
from django.utils import simplejson as json

from respite.utils import parse_content_type

class HttpMethodOverrideMiddleware:
    """
    Facilitate for overriding the HTTP method with the X-HTTP-Method-Override
    header or a '_method' HTTP POST parameter.
    """

    def process_request(self, request):
        if 'HTTP_X_HTTP_METHOD_OVERRIDE' in request.META \
        or '_method' in request.POST:
            request.method = (
                request.META.get('HTTP_X_HTTP_METHOD_OVERRIDE') or
                request.POST.get('_method')
            ).upper()

            # Proxy the CSRF middleware token to a header for compatibility with non-idempotent
            # methods besides POST.
            if 'csrfmiddlewaretoken' in request.POST:
                request.META.setdefault('HTTP_X_CSRFTOKEN', request.POST['csrfmiddlewaretoken'])

            request.POST = QueryDict('')

class HttpPutMiddleware:
    """
    Facilitate for HTTP PUT in the same way Django facilitates for HTTP GET
    and HTTP POST; populate a QueryDict instance with the request body in request.PUT.
    """

    def process_request(self, request):
        if request.method == 'PUT':
            request.PUT = QueryDict(request.raw_post_data)

class HttpPatchMiddleware:
    """
    Facilitate for HTTP PATCH in the same way Django facilitates for HTTP GET
    and HTTP POST; populate a QueryDict instance with the request body in request.PATCH.
    """

    def process_request(self, request):
        if request.method == 'PATCH':
            request.PATCH = QueryDict(request.raw_post_data)

class JsonMiddleware:
    """
    Parse JSON in POST, PUT and PATCH requests.
    """

    def process_request(self, request):
        if 'CONTENT_TYPE' in request.META:
            content_type, encoding = parse_content_type(request.META['CONTENT_TYPE'])

            if content_type == 'application/json':
                data = json.loads(request.raw_post_data, encoding)

                if request.method in ['POST', 'PUT', 'PATCH']:
                    setattr(request, request.method, QueryDict(urlencode(data)))
